#include <lv2/lv2.h>
#include <lv2/libc.h>
#include <lv2/interrupt.h>
#include <lv2/modules.h>
#include <lv2/process.h>
#include <lv2/memory.h>
#include <lv2/io.h>
#include <lv2/symbols.h>
#include <lv2/error.h>
#include <lv2/thread.h>
#include <lv2/syscall.h>
#include <lv2/debug.h>
#include "common.h"
#include "modulespatch.h"
#include "syscall8.h"

#define MAX_VSH_PLUGINS		6

sys_prx_id_t vsh_plugins[MAX_VSH_PLUGINS];
static int loading_vsh_plugin;

int sys_prx_load_vsh_plugin(process_t vsh_process, unsigned int slot, char *path, void *arg, uint32_t arg_size)
{
	void *kbuf, *vbuf;
	sys_prx_id_t prx;
	int ret;

	path = get_secure_user_ptr(path);
	arg = get_secure_user_ptr(arg);

	if (slot >= MAX_VSH_PLUGINS || (arg != NULL && arg_size > KB(64)))
		return EINVAL;

	if (vsh_plugins[slot] != 0)
	{
		return EKRESOURCE;
	}

	//loading_vsh_plugin = 1;
	prx = prx_load_module(vsh_process, 0, 0, path);
	//loading_vsh_plugin  = 0;

	if (prx < 0)
	{
		printf("prx_load_module returned: %llx\n", (uint64_t) prx);
		return prx;
	}

	if (arg && arg_size > 0)
	{
		page_allocate_auto(vsh_process, KB(64), 0x2F, &kbuf);
		page_export_to_proc(vsh_process, kbuf, 0x40000, &vbuf);
		copy_from_user(arg, kbuf, arg_size);
	}
	else
	{
		vbuf = NULL;
	}

	ret = prx_start_module_with_thread(prx, vsh_process, 0, (uint64_t)vbuf);

	if (vbuf)
	{
		page_unexport_from_proc(vsh_process, vbuf);
		page_free(vsh_process, kbuf, 0x2F);
	}

	if (ret == 0)
	{
		vsh_plugins[slot] = prx;
	}
	else
	{
		printf("prx_start_module_with_thread returned: %llx\n", ret);
		prx_stop_module_with_thread(prx, vsh_process, 0, 0);
		prx_unload_module(prx, vsh_process);
	}

	//DPRINTF("Vsh plugin load: %x\n", ret);

	return ret;
}

int sys_prx_unload_vsh_plugin(process_t vsh_process, unsigned int slot)
{
	int ret;
	sys_prx_id_t prx;

	//DPRINTF("Trying to unload vsh plugin %x\n", slot);

	if (slot >= MAX_VSH_PLUGINS)
		return EINVAL;

	prx = vsh_plugins[slot];
	//DPRINTF("Current plugin: %08X\n", prx);

	if (prx == 0)
		return ENOENT;

	ret = prx_stop_module_with_thread(prx, vsh_process, 0, 0);
	if (ret == 0)
	{
		ret = prx_unload_module(prx, vsh_process);
	}
	else
	{
		//DPRINTF("Stop failed: %x!\n", ret);
	}

	if (ret == 0)
	{
		vsh_plugins[slot] = 0;
		//DPRINTF("Vsh plugin unloaded succesfully!\n");
	}
	else
	{
		//DPRINTF("Unload failed : %x!\n", ret);
	}

	return ret;
}

